from django.db import models


class BaseModel(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, null=False)
    updated_at = models.DateTimeField(auto_now=True, null=False)

    class Meta:
        abstract = True



def raise_(e):
    """A helper to raise an exception when you can't call raise directly.

    E.g. raising an exception in a lambda function. get(some_key, lambda: _raise(Exception("Some message")))
    """
    raise e



import logging
from typing import Union

from django.conf import settings
from django.core import mail
from django.core.mail import EmailMessage

logger = logging.getLogger(__name__)


def send_mail(
    to_email: Union[list, str],
    template_id: str,
    data: dict = {},
    from_email: str = None,
    esp_extra: dict = None,  # https://anymail.readthedocs.io/en/stable/esps/sendgrid/#esp-extra-support)
):
    """"""

    if isinstance(to_email, str):
        to_email = [to_email]
    logger.info(f"Prepping email(s) to: {to_email} using template_id {template_id}")

    num_sent = 0
    connection = mail.get_connection()
    d = {}
    messages = []
    for t in to_email:
        if getattr(
            settings, "APP_ENVIRONMENT", None
        ) and settings.APP_ENVIRONMENT not in (
            "production",
            "prod",
        ):
            """override data and to_email"""
            logger.info(
                f"Not in production. Replacing to email {t} with {settings.SERVER_EMAIL}"
            )
            d[settings.SERVER_EMAIL] = data.get(t, {})
            to = settings.SERVER_EMAIL
        else:
            d = {}
            d[t] = data.get(t)
            to = t
        message = (
            EmailMessage(to=[to])
            if not from_email
            else EmailMessage(to=[to], from_email=from_email)
        )
        if esp_extra:
            message.esp_extra = esp_extra
        message.template_id = template_id
        message.merge_data = d.copy()
        messages.append(message)

    num_sent = connection.send_messages(messages)
    logger.info(f"Send {num_sent} emails with template id {template_id}")
    connection.close()
    return num_sent



import json
from django.http import HttpResponse

from rest_framework import status as rest_status


class ErrorPagesMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        if "application/json" in request.content_type:
            if response.status_code == rest_status.HTTP_404_NOT_FOUND:
                data = {"detail": "{0} not found".format(request.path)}
                status = rest_status.HTTP_404_NOT_FOUND
            elif response.status_code == rest_status.HTTP_403_FORBIDDEN:
                data = {"detail": "Forbidden"}
                status = rest_status.HTTP_403_FORBIDDEN
            elif response.status_code == rest_status.HTTP_500_INTERNAL_SERVER_ERROR:
                data = {"detail": "Internal Server Error"}
                status = rest_status.HTTP_500_INTERNAL_SERVER_ERROR
            else:
                return response

            return HttpResponse(
                json.dumps(data), content_type="application/json", status=status
            )

        return response



from django.db import models


class EmailField(models.EmailField):

    description = "A email used for user accounts (always lowercase)."

    def __init__(self, *args, **kwargs):
        super(EmailField, self).__init__(*args, **kwargs)

    def get_prep_value(self, value):
        if value is not None:
            return value.lower()
        return value



import os
import io
import sys
from setuptools import find_packages, setup, Command
from shutil import rmtree

NAME = "django-crawfish"
DESCRIPTION = "pending..."
URL = "https://gitlab.com/crawfordleeds/crawfish"
EMAIL = "crawford@crawfordleeds.com"
AUTHOR = "Crawford Leeds"
REQUIRES_PYTHON = ">=3.6"

# Dont' manually change the version here. Use bumpversion instead
VERSION = "0.0.10"

# Required packages for this module
REQUIRED = ["Django>=2.2"]

# Optional packages
EXTRAS = {
    "dev": [
        "black==20.8b1",
        "bumpversion==0.6.0",  # This will also install bump2version
        "tox==3.20.1",
        "coverage==5.3",
        "twine==3.2.0",
        "djangorestframework==3.12.2",
    ]
}

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long description
try:
    with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
        long_description = f"\n{f.read()}"
except FileNotFoundError:
    long_description = DESCRIPTION


class UploadCommand(Command):
    """Support setup.py upload"""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Print in bold"""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds...")
            rmtree(os.path.join(here, "dist"))
        except OSError:
            pass

        self.status("Build source and wheel (universal) distribution...")
        os.system("{0} setup.py sdist bdist_wheel --universal".format(sys.executable))

        self.status("Upload the package to PyPI via Twine...")
        os.system(
            f"twine upload dist/*"
        )  # twine username/password set in .pypirc file or TWINE_USERNAME / TWINE_PASSWORD env variables


setup(
    name=NAME,
    version=VERSION,  # Don't change this manually, use bumpversion instead,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests", "tests.*"]),
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license="MIT",
    url=URL,
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
    ],
    cmdclass={"upload": UploadCommand},
)



from rest_framework.response import Response


def get_paginated_response(
    *, pagination_class, serializer_class, queryset, request, view
):
    paginator = pagination_class()

    page = paginator.paginate_queryset(queryset, request, view=view)

    if page is not None:
        serializer = serializer_class(page, many=True)
        return paginator.get_paginated_response(serializer.data)

    serializer = serializer_class(queryset, many=True)

    return Response(data=serializer.data)
